@@ -98,7 +98,7 @@ static void cb_xfce_backdrop_image_files_changed(GFileMonitor     *monitor,
                                                  GFileMonitorEvent event,
                                                  gpointer          user_data);
 
-struct _XfceBackdropPriv
+struct _XfceBackdropPrivate
 {
     gint width, height;
     gint bpp;
@@ -187,7 +187,7 @@ create_solid(GdkRGBA *color,
 
     cairo_destroy(cr);
     cairo_surface_destroy(surface);
-    
+
     return pix;
 }
 
@@ -361,6 +361,11 @@ cb_xfce_backdrop_image_files_changed(GFileMonitor     *monitor,
                 backdrop->priv->image_files = g_list_delete_link(backdrop->priv->image_files, item);
 
             g_free(changed_file);
+
+            if (backdrop->priv->cycle_timer_id) {
+                g_source_remove(backdrop->priv->cycle_timer_id);
+                backdrop->priv->cycle_timer_id = 0;
+            }
             break;
         case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
             changed_file = g_file_get_path(file);
@@ -650,20 +655,18 @@ xfce_backdrop_choose_chronological(XfceBackdrop *backdrop)
 /* gobject-related functions */
 
 
-G_DEFINE_TYPE(XfceBackdrop, xfce_backdrop, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(XfceBackdrop, xfce_backdrop, G_TYPE_OBJECT)
 
 
 static void
 xfce_backdrop_class_init(XfceBackdropClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
-    
-    g_type_class_add_private(klass, sizeof(XfceBackdropPriv));
-    
+
     gobject_class->finalize = xfce_backdrop_finalize;
     gobject_class->set_property = xfce_backdrop_set_property;
     gobject_class->get_property = xfce_backdrop_get_property;
-    
+
     backdrop_signals[BACKDROP_CHANGED] = g_signal_new("changed",
             G_OBJECT_CLASS_TYPE(gobject_class), G_SIGNAL_RUN_FIRST,
             G_STRUCT_OFFSET(XfceBackdropClass, changed), NULL, NULL,
@@ -738,7 +741,7 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
                                                       "backdrop-cycle-period",
                                                       "backdrop-cycle-period",
                                                       XFCE_TYPE_BACKDROP_CYCLE_PERIOD,
-                                                      XFCE_BACKDROP_PERIOD_MINUES,
+                                                      XFCE_BACKDROP_PERIOD_MINUTES,
                                                       XFDESKTOP_PARAM_FLAGS));
 
     g_object_class_install_property(gobject_class, PROP_BACKDROP_CYCLE_TIMER,
@@ -761,8 +764,7 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
 static void
 xfce_backdrop_init(XfceBackdrop *backdrop)
 {
-    backdrop->priv = G_TYPE_INSTANCE_GET_PRIVATE(backdrop, XFCE_TYPE_BACKDROP,
-                                                 XfceBackdropPriv);
+    backdrop->priv = xfce_backdrop_get_instance_private(backdrop);
     backdrop->priv->cycle_timer_id = 0;
 
     /* color defaults */
@@ -780,9 +782,9 @@ static void
 xfce_backdrop_finalize(GObject *object)
 {
     XfceBackdrop *backdrop = XFCE_BACKDROP(object);
-    
+
     g_return_if_fail(backdrop != NULL);
-    
+
     if(backdrop->priv->image_path)
         g_free(backdrop->priv->image_path);
 
@@ -930,12 +932,12 @@ XfceBackdrop *
 xfce_backdrop_new(GdkVisual *visual)
 {
     XfceBackdrop *backdrop;
-    
+
     g_return_val_if_fail(GDK_IS_VISUAL(visual), NULL);
-    
+
     backdrop = g_object_new(XFCE_TYPE_BACKDROP, NULL);
     backdrop->priv->bpp = gdk_visual_get_depth(visual);
-    
+
     return backdrop;
 }
 
@@ -954,11 +956,11 @@ xfce_backdrop_new_with_size(GdkVisual *visual,
                             gint height)
 {
     XfceBackdrop *backdrop;
-    
+
     g_return_val_if_fail(GDK_IS_VISUAL(visual), NULL);
-    
+
     backdrop = g_object_new(XFCE_TYPE_BACKDROP, NULL);
-    
+
     backdrop->priv->bpp = gdk_visual_get_depth(visual);
     backdrop->priv->width = width;
     backdrop->priv->height = height;
@@ -1036,7 +1038,7 @@ xfce_backdrop_set_first_color(XfceBackdrop *backdrop,
                               const GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color != NULL);
-    
+
     if(color->red != backdrop->priv->color1.red
             || color->green != backdrop->priv->color1.green
             || color->blue != backdrop->priv->color1.blue
@@ -1057,7 +1059,7 @@ xfce_backdrop_get_first_color(XfceBackdrop *backdrop,
                               GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color);
-    
+
     memcpy(color, &backdrop->priv->color1, sizeof(GdkRGBA));
 }
 
@@ -1076,7 +1078,7 @@ xfce_backdrop_set_second_color(XfceBackdrop *backdrop,
                                const GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color != NULL);
-    
+
     if(color->red != backdrop->priv->color2.red
             || color->green != backdrop->priv->color2.green
             || color->blue != backdrop->priv->color2.blue
@@ -1098,7 +1100,7 @@ xfce_backdrop_get_second_color(XfceBackdrop *backdrop,
                                GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color);
-    
+
     memcpy(color, &backdrop->priv->color2, sizeof(GdkRGBA));
 }
 
@@ -1182,7 +1184,7 @@ xfce_backdrop_set_image_filename(XfceBackdrop *backdrop, const gchar *filename)
 
     /* Now we can free the old path and setup the new one */
     g_free(backdrop->priv->image_path);
-    
+
     if(filename)
         backdrop->priv->image_path = g_strdup(filename);
     else
@@ -1354,7 +1356,7 @@ xfce_backdrop_set_cycle_timer(XfceBackdrop *backdrop, guint cycle_timer)
                 cycle_interval = backdrop->priv->cycle_timer;
                 break;
 
-            case XFCE_BACKDROP_PERIOD_MINUES:
+            case XFCE_BACKDROP_PERIOD_MINUTES:
                 cycle_interval = backdrop->priv->cycle_timer * 60;
                 break;
 
@@ -1534,10 +1536,7 @@ xfce_backdrop_force_cycle(XfceBackdrop *backdrop)
 
     TRACE("entering");
 
-    /* force it to update */
-    xfce_backdrop_cycle_backdrop(backdrop);
-
-    /* Update the timer, if running */
+    /* Just update the timer, if running, to cycle the backdrop */
     xfce_backdrop_timer(backdrop);
 }
 
@@ -1711,10 +1710,8 @@ xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
         return;
     }
 
-    /* invalid backdrop? quit */
+    /* invalid backdrop? quit but don't free image data */
     if(!XFCE_IS_BACKDROP(backdrop)) {
-        xfce_backdrop_image_data_release(image_data);
-        g_free(image_data);
         return;
     }
 
@@ -1782,12 +1779,15 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
     gint dx, dy, xo, yo;
     gdouble xscale, yscale;
     GdkInterpType interp;
+    gboolean rotated = FALSE;
 
     TRACE("entering");
 
-    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+    /* invalid backdrop? just quit */
+    if(!XFCE_IS_BACKDROP(backdrop))
+        return;
 
-    /* canceled? quit now */
+    /* canceled? free data and quit now */
     if(g_cancellable_is_cancelled(image_data->cancellable)) {
         xfce_backdrop_image_data_release(image_data);
         g_free(image_data);
@@ -1796,6 +1796,8 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
     image = gdk_pixbuf_loader_get_pixbuf(loader);
     if(image) {
+        gint iw_orig = gdk_pixbuf_get_width(image);
+
         /* If the image is supposed to be rotated, do that now */
         GdkPixbuf *temp = gdk_pixbuf_apply_embedded_orientation (image);
         /* Do not unref image, gdk_pixbuf_loader_get_pixbuf is transfer none */
@@ -1803,6 +1805,8 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
         iw = gdk_pixbuf_get_width(image);
         ih = gdk_pixbuf_get_height(image);
+
+        rotated = (iw_orig != iw);
     }
 
     if(backdrop->priv->width == 0 || backdrop->priv->height == 0) {
@@ -1812,14 +1816,14 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
         w = backdrop->priv->width;
         h = backdrop->priv->height;
     }
-    
+
     istyle = backdrop->priv->image_style;
-    
+
     /* if the image is the same as the screen size, there's no reason to do
      * any scaling at all */
     if(w == iw && h == ih)
         istyle = XFCE_BACKDROP_IMAGE_CENTERED;
-    
+
     /* if we don't need to do any scaling, don't do any interpolation.  this
      * fixes a problem where hyper/bilinear filtering causes blurriness in
      * some images.  http://bugzilla.xfce.org/show_bug.cgi?id=2939 */
@@ -1853,6 +1857,9 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
         return;
     }
 
+    xscale = (gdouble)w / iw;
+    yscale = (gdouble)h / ih;
+
     switch(istyle) {
         case XFCE_BACKDROP_IMAGE_NONE:
             /* do nothing */
@@ -1867,7 +1874,7 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
                     MIN(w, iw), MIN(h, ih), xo, yo, 1.0, 1.0,
                     interp, 255);
             break;
-        
+
         case XFCE_BACKDROP_IMAGE_TILED:
             tmp = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, w, h);
             /* Now that the image has been loaded, recalculate the image
@@ -1880,7 +1887,7 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
                 for(j = 0; (j * ih) < h; j++) {
                     gint newx = iw * i, newy = ih * j;
                     gint neww = iw, newh = ih;
-                    
+
                     if((newx + neww) > w)
                         neww = w - newx;
                     if((newy + newh) > h)
@@ -1890,20 +1897,21 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
                             neww, newh, tmp, newx, newy);
                 }
             }
-            
+
             gdk_pixbuf_composite(tmp, final_image, 0, 0, w, h,
                     0, 0, 1.0, 1.0, interp, 255);
             g_object_unref(G_OBJECT(tmp));
             break;
-        
+
         case XFCE_BACKDROP_IMAGE_STRETCHED:
             gdk_pixbuf_composite(image, final_image, 0, 0, w, h,
-                    0, 0, 1, 1, interp, 255);
+                    0, 0,
+                    rotated ? xscale : 1,
+                    rotated ? yscale : 1,
+                    interp, 255);
             break;
-        
+
         case XFCE_BACKDROP_IMAGE_SCALED:
-            xscale = (gdouble)w / iw;
-            yscale = (gdouble)h / ih;
             if(xscale < yscale) {
                 yscale = xscale;
                 xo = 0;
@@ -1917,14 +1925,14 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
             dy = yo;
 
             gdk_pixbuf_composite(image, final_image, dx, dy,
-                    iw * xscale, ih * yscale, xo, yo, 1, 1,
+                    iw * xscale, ih * yscale, xo, yo,
+                    rotated ? xscale : 1,
+                    rotated ? yscale : 1,
                     interp, 255);
             break;
-        
+
         case XFCE_BACKDROP_IMAGE_ZOOMED:
         case XFCE_BACKDROP_IMAGE_SPANNING_SCREENS:
-            xscale = (gdouble)w / iw;
-            yscale = (gdouble)h / ih;
             if(xscale < yscale) {
                 xscale = yscale;
                 xo = (w - (iw * xscale)) * 0.5;
@@ -1936,9 +1944,12 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
             }
 
             gdk_pixbuf_composite(image, final_image, 0, 0,
-                    w, h, xo, yo, 1, 1, interp, 255);
+                    w, h, xo, yo,
+                    rotated ? xscale : 1,
+                    rotated ? yscale : 1,
+                    interp, 255);
             break;
-        
+
         default:
             g_critical("Invalid image style: %d\n", (gint)istyle);
     }
